/*
 * Copyright (c) 2009 The openGion Project.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.opengion.plugin.query;

import org.opengion.hayabusa.db.AbstractQuery;
import org.opengion.hayabusa.db.DBErrMsg;
import org.opengion.hayabusa.common.HybsSystemException;
import org.opengion.fukurou.util.ErrorMessage;
import org.opengion.fukurou.util.StringUtil;

import java.sql.Connection;
import java.sql.CallableStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.sql.Array;								// 6.0.0.0 (2014/04/11) Oracle11g(11.2.0.3のドライバ) 対応。oracle.sql.ARRAY の置き換え
import oracle.jdbc.OracleConnection;				// 6.0.0.0 (2014/04/11) Oracle11g(11.2.0.3のドライバ) 対応

import oracle.jdbc.OracleTypes;						// CURSOR が残る
import oracle.jdbc.OracleCallableStatement;			// CURSOR が残る

import java.util.Map;

/**
 * エントリ系 PL/SQL をコールして、結果カーソルから、DBTableModel を作成します。
 *
 * java.sql.CallableStatement を用いて、データベース検索処理を行います。
 * 引数を配列指定で渡すことが出来，エラー時には、DBErrMsg オブジェクトにエラー情報を
 * 格納して返すことが可能です。
 * 内部変数の受け渡しのデフォルト実装は、AbstractQuery クラスを継承している
 * ため,ここでは、execute() メソッドを実装しています。
 * このクラスでは、ステートメント文を execute() する事により,データベースを
 * 検索した結果を DBTableModel に割り当てます。
 *
 * @og.formSample
 * 例：jsp/TYPESB/result.jsp  (検索系：カーソル返し)
 * 例：jsp/TYPE3B/entry.jsp   (エントリ系)
 *     names には、ARG_ARRAY 配列に順次セットされます。
 *     使用する場合は、一旦わかり易い変数に受けて利用してください。
 *     呼び出す PL/SQL では、検索系PL/SQL です。(下記の例は、エントリ系)
 *
 * &lt;og:query
 *     command    = &quot;NEW&quot;
 *     names      = &quot;SYSTEM_ID,LANG,CLM,NAME_JA,LABEL_NAME,KBSAKU,USER.ID&quot;
 *     queryType  = &quot;JDBCErrMsg&quot;
 *     displayMsg = &quot;&quot; &gt;
 *        {call TYPE3B01.TYPE3B01(?,?,?,?)}
 * &lt;/og:query&gt;
 *
 *    CREATE OR REPLACE PACKAGE TYPE3B01 AS
 *        TYPE CUST_CURSOR IS REF CURSOR;
 *        PROCEDURE TYPE3B01(
 *            P_KEKKA      OUT    NUMBER,
 *            P_ERRMSGS    OUT    ERR_MSG_ARRAY,
 *            P_RC1        OUT    CUST_CURSOR,
 *            P_ARGS       IN     ARG_ARRAY     );
 *    END;
 *
 *     P_SYSTEM_ID     GEA08.SYSTEM_ID%TYPE     := P_ARGS(1);    --システムＩＤ
 *     P_LANG          GEA08.LANG%TYPE          := P_ARGS(2);    --言語
 *     P_CLM           GEA08.CLM%TYPE           := P_ARGS(3);    --項目
 *     P_NAME_JA       GEA08.NAME_JA%TYPE       := P_ARGS(4);    --名称(漢字)
 *     P_LABEL_NAME    GEA08.LABEL_NAME%TYPE    := P_ARGS(5);    --表示名称
 *     P_KBSAKU        GEA08.KBSAKU%TYPE        := P_ARGS(6);    --作成区分
 *     P_USRSET        GEA08.USRSET%TYPE        := P_ARGS(7);    --登録者
 *
 * @og.group データ表示
 * @og.group データ編集
 *
 * @version  4.0
 * @author   Kazuhiko Hasegawa
 * @since    JDK5.0,
 */
public class Query_JDBCErrMsg extends AbstractQuery {
	/** このプログラムのVERSION文字列を設定します。	{@value} */
	private static final String VERSION = "6.9.3.0 (2018/03/26)" ;

	/**
	 * デフォルトコンストラクター
	 *
	 * @og.rev 6.4.2.0 (2016/01/29) PMD refactoring. Each class should declare at least one constructor.
	 */
	public Query_JDBCErrMsg() { super(); }		// これも、自動的に呼ばれるが、空のメソッドを作成すると警告されるので、明示的にしておきます。

	/**
	 * 引数配列付のクエリーを実行します。
	 * 処理自体は, #execute() と同様に、各サブクラスの実装に依存します。
	 * これは、CallableStatement を用いて、データベース検索処理を行います。
	 * {call TYPE3B01.TYPE3B01(?,?,?,?)} で、４番目の引数には、
	 * names で指定したリクエスト情報が、ARG_ARRAY 配列に順次セットされます。
	 * 使用する場合は、一旦わかり易い変数に受けて利用してください。
	 * 呼び出す PL/SQL では、検索系PL/SQL です。
	 *
	 * @og.rev 2.3.1.3 (2003/01/28) Open Cursor が、大量に残る件の対応。ResultSet を close()
	 * @og.rev 3.1.1.0 (2003/03/28) 同期メソッド(synchronized付き)を非同期に変更する。
	 * @og.rev 3.3.3.1 (2003/07/18) ＤＢ登録時の後ろスペースを削除する。
	 * @og.rev 3.5.2.0 (2003/10/20) 内部オブジェクトタイプ名を システムパラメータ で定義します。
	 * @og.rev 3.5.6.0 (2004/06/18) nullに対する無駄な比較を削除します。
	 * @og.rev 3.8.0.8 (2005/10/03) エラーメッセージの出力順をメッセージ＋Queryに変更します。
	 * @og.rev 6.0.0.0 (2014/04/11) Oracle11g(11.2.0.3のドライバ)対応
	 * @og.rev 6.3.6.1 (2015/08/28) close(),realClose() 廃止。Queryはキャッシュしません。
	 * @og.rev 6.4.2.1 (2016/02/05) try-with-resources 文で記述。
	 * @og.rev 6.9.3.0 (2018/03/26) DB_FETCH_SIZE追加。
	 *
	 * @param   args オブジェクトの引数配列(可変長引数)
	 */
	@Override
	public void execute( final String... args ) {			// 6.1.1.0 (2015/01/17) refactoring
		try {
			final Connection conn = getConnection();
			// 6.4.2.1 (2016/02/05) try-with-resources 文
			try( CallableStatement callStmt = conn.prepareCall( getStatement() ) ) {
				callStmt.setQueryTimeout( DB_MAX_QUERY_TIMEOUT );
				callStmt.setFetchSize( DB_FETCH_SIZE );			// 6.9.3.0 (2018/03/26)

				final Map<String,Class<?>> map = conn.getTypeMap();
				map.put( ERR_MSG,DBErrMsg.class );	// 4.0.0 (2005/01/31)

				// 6.0.0.0 (2014/04/11) Oracle11g(11.2.0.3のドライバ)対応 http://docs.oracle.com/cd/E28389_01/web.1111/b60995/thirdparty.htm
				final Array newArray = ((OracleConnection)conn).createOracleArray( ARG_ARRAY, StringUtil.rTrims( args ));		// 6.0.0.0 (2014/04/11) Oracle11g(11.2.0.3のドライバ)対応

				callStmt.registerOutParameter(1, Types.INTEGER);
				callStmt.registerOutParameter(2, Types.ARRAY,ERR_MSG_ARRAY);			// 6.0.0.0 (2014/04/11) Oracle11g(11.2.0.3のドライバ)対応
				callStmt.registerOutParameter(3, OracleTypes.CURSOR);
				callStmt.setArray( 4,newArray );										// 6.0.0.0 (2014/04/11) Oracle11g(11.2.0.3のドライバ)対応

				callStmt.execute();

				final int rtnCode = callStmt.getInt(1);
				setErrorCode( rtnCode );
				if( rtnCode < ErrorMessage.NG ) {		// 異常以外の場合
					try( final ResultSet resultSet = ((OracleCallableStatement)callStmt).getCursor(3) ) {
						createTableModel( resultSet );
					}
				}
				if( rtnCode > ErrorMessage.OK ) {		// 正常以外の場合
					final Array rtn3 = callStmt.getArray(2);									// 6.0.0.0 (2014/04/11) Oracle11g(11.2.0.3のドライバ)対応
					final Object[] rtnval3 = (Object[])rtn3.getArray();
					final ErrorMessage errMessage = new ErrorMessage( "Query_JDBCErrMsg Error!!" );
					for( int i=0; i<rtnval3.length; i++ ) {
						final DBErrMsg er = (DBErrMsg)rtnval3[i];
						if( er == null ) { break; }
						errMessage.addMessage( er.getErrMsg() );
					}
					setErrorMessage( errMessage );
				}
			}
		}
		catch( final SQLException ex) {
			setErrorCode( ErrorMessage.EXCEPTION );
			final String errMsg = ex.getMessage() + ":" + ex.getSQLState() + CR
						+ getStatement() + CR;
			throw new HybsSystemException( errMsg,ex );		// 3.5.5.4 (2004/04/15) 引数の並び順変更
		}
	}
}
